@@ -9,6 +9,7 @@ use Firefox::Marionette::Window::Rect();
 use Firefox::Marionette::Element::Rect();
 use Firefox::Marionette::Timeouts();
 use Firefox::Marionette::Capabilities();
+use Firefox::Marionette::Profile();
 use JSON();
 use Socket();
 use English qw( -no_match_vars );
@@ -28,7 +29,7 @@ BEGIN {
     }
 }
 
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 
 sub _ANYPROCESS                    { return -1 }
 sub _COMMAND                       { return 0 }
@@ -51,11 +52,11 @@ sub new {
         $self->{firefox_binary} = $parameters{firefox_binary};
     }
     if ( $parameters{profile_name} ) {
-        $self->{profile} = $parameters{profile_name};
+        $self->{profile_name} = $parameters{profile_name};
         push @arguments, ( '-P', $self->{profile_name} );
     }
     else {
-        my $profile_directory = $self->_setup_new_profile();
+        my $profile_directory = $self->_setup_new_profile($parameters{profile});
         push @arguments,
           ( '-profile', $profile_directory, '--no-remote', '--new-instance' );
     }
@@ -71,6 +72,17 @@ sub new {
     return $self;
 }
 
+sub profile {
+    my ( $self, $new ) = @_;
+    my $old =
+      Firefox::Marionette::Profile->parse(
+        File::Spec->catfile( $self->{profile_path} ) );
+    if ( defined $new ) {
+        $new->save( $self->{profile_path} );
+    }
+    return $old;
+}
+
 sub _pid {
     my ($self) = @_;
     return $self->{_pid};
@@ -199,7 +211,7 @@ sub _setup_local_connection_to_firefox {
 }
 
 sub _setup_new_profile {
-    my ($self) = @_;
+    my ($self, $profile) = @_;
     my $profile_directory = File::Temp->newdir(
         File::Spec->catdir(
             File::Spec->tmpdir(), 'firefox_marionette_XXXXXXXXXX'
@@ -208,36 +220,16 @@ sub _setup_new_profile {
     $self->{profile_directory} = $profile_directory;
     $self->{profile_path} =
       File::Spec->catfile( $profile_directory, 'prefs.js' );
-    my $profile_handle =
-      FileHandle->new( $self->{profile_path},
-        Fcntl::O_WRONLY() | Fcntl::O_CREAT() | Fcntl::O_EXCL(),
-        Fcntl::S_IRWXU() )
-      or Carp::croak(
-        "Failed to open '$self->{profile_path}' for writing:$EXTENDED_OS_ERROR"
-      );
-    $self->_write_profile_contents($profile_handle)
-      or Carp::croak(
-        "Failed to write to '$self->{profile_path}':$EXTENDED_OS_ERROR");
-    $profile_handle->close()
-      or
-      Carp::croak("Failed to close '$self->{profile_path}':$EXTENDED_OS_ERROR");
+    if ($profile) {
+	} else {
+	    my $port = _ANY_PORT();
+		$profile = Firefox::Marionette::Profile->new();
+	    $profile->set_value('marionette.port', $port);
+	}
+    $profile->save($self->{profile_path});
     return $profile_directory;
 }
 
-sub _write_profile_contents {
-    my ( $self, $profile_handle ) = @_;
-    my $port = _ANY_PORT();
-    return $profile_handle->print(<<"_PROFILE_");
-user_pref("browser.startup.homepage", "about:blank");
-user_pref("browser.startup.homepage_override.mstone", "ignore");
-user_pref("startup.homepage_welcome_url", "about:blank");
-user_pref("startup.homepage_welcome_url.additional", "about:blank");
-user_pref("xpinstall.signatures.required", false);
-user_pref("toolkit.telemetry.reportingpolicy.firstRun", false);
-user_pref("marionette.port", $port);
-_PROFILE_
-}
-
 sub _get_port {
     my ($self) = @_;
     my $port;
@@ -338,7 +330,7 @@ sub find_elements {
     );
     my $response = $self->_get_response($message_id);
     return
-      map { Firefox::Marionette::Element->new( $self, $_ ) }
+      map { Firefox::Marionette::Element->new( $self, %{$_} ) }
       @{ $response->result() };
 }
 
@@ -738,7 +730,7 @@ sub active_element {
         [ _COMMAND(), $message_id, 'WebDriver:GetActiveElement' ] );
     my $response = $self->_get_response($message_id);
     return Firefox::Marionette::Element->new( $self,
-        $response->result()->{value} );
+        %{$response->result()->{value}} );
 }
 
 sub uri {
@@ -938,7 +930,7 @@ sub find_element {
     );
     my $response = $self->_get_response($message_id);
     return Firefox::Marionette::Element->new( $self,
-        $response->result()->{value} );
+        %{ $response->result()->{value} } );
 }
 
 sub active_frame {
@@ -949,7 +941,7 @@ sub active_frame {
     my $response = $self->_get_response($message_id);
     if ( defined $response->result()->{value} ) {
         return Firefox::Marionette::Element->new( $self,
-            $response->result()->{value} );
+            %{ $response->result()->{value} } );
     }
     else {
         return;
@@ -1103,6 +1095,19 @@ sub window_type {
     return $response->result()->{value};
 }
 
+sub switch_to_shadow_root {
+    my ( $self, $element ) = @_;
+    my $message_id = $self->_new_message_id();
+    $self->_send_request(
+        [
+            _COMMAND(), $message_id,
+            'WebDriver:SwitchToShadowRoot', { element => $element->uuid() }
+        ]
+    );
+    my $response = $self->_get_response($message_id);
+    return $self;
+}
+
 sub switch_to_frame {
     my ( $self, $element ) = @_;
     my $message_id = $self->_new_message_id();
@@ -1154,9 +1159,9 @@ my $x = <<'_DOC_';
 #  "WebDriver:GetAlertText": GeckoDriver.prototype.getTextFromDialog,
 #  "WebDriver:GetCapabilities": GeckoDriver.prototype.getSessionCapabilities,
 #  "WebDriver:GetChromeWindowHandle": GeckoDriver.prototype.getChromeWindowHandle,
-  "WebDriver:GetChromeWindowHandles": GeckoDriver.prototype.getChromeWindowHandles,
+#  "WebDriver:GetChromeWindowHandles": GeckoDriver.prototype.getChromeWindowHandles,
 #  "WebDriver:GetCookies": GeckoDriver.prototype.getCookies,
-  "WebDriver:GetCurrentChromeWindowHandle": GeckoDriver.prototype.getChromeWindowHandle,
+#  "WebDriver:GetCurrentChromeWindowHandle": GeckoDriver.prototype.getChromeWindowHandle,
 #  "WebDriver:GetCurrentURL": GeckoDriver.prototype.getCurrentUrl,
 #  "WebDriver:GetElementAttribute": GeckoDriver.prototype.getElementAttribute,
 #  "WebDriver:GetElementCSSValue": GeckoDriver.prototype.getElementValueOfCssProperty,
@@ -1187,8 +1192,8 @@ my $x = <<'_DOC_';
 #  "WebDriver:SetTimeouts": GeckoDriver.prototype.setTimeouts,
 #  "WebDriver:SetWindowRect": GeckoDriver.prototype.setWindowRect,
 #  "WebDriver:SwitchToFrame": GeckoDriver.prototype.switchToFrame,
-  "WebDriver:SwitchToParentFrame": GeckoDriver.prototype.switchToParentFrame,
-  "WebDriver:SwitchToShadowRoot": GeckoDriver.prototype.switchToShadowRoot,
+#  "WebDriver:SwitchToParentFrame": GeckoDriver.prototype.switchToParentFrame,
+#  "WebDriver:SwitchToShadowRoot": GeckoDriver.prototype.switchToShadowRoot,
   "WebDriver:SwitchToWindow": GeckoDriver.prototype.switchToWindow,
 #  "WebDriver:TakeScreenshot": GeckoDriver.prototype.takeScreenshot,
 _DOC_
@@ -1288,7 +1293,7 @@ Firefox::Marionette - Automate the Firefox browser with the Marionette protocol
 
 =head1 VERSION
 
-Version 0.09
+Version 0.10
 
 =head1 SYNOPSIS
 
@@ -1323,6 +1328,8 @@ accepts an optional hash as a parameter.  Allowed keys are below;
 
 =item * profile_name - pick a specific existing profile to automate, rather than creating a new profile.  Note that L<firefox|https://firefox.com> refuses to allow more than one instance of a profile to run at the same time.
 
+=item * profile - create a new profile based on the supplied profile.
+
 =back
 
 This method returns a new C<Firefox::Marionette> object, connected to an instance of L<firefox|https://firefox.com>
@@ -1367,6 +1374,10 @@ returns the elements in the current browsing context that match the search param
 
 This method is subject to the L<implicit|Firefox::Marionette::Timeouts#implicit> timeout.
 
+=head2 profile
+
+accepts a L<profile|Firefox::Marionette::Profile> as an optional parameter and saves this as the new profile.  It returns the previous (or current) L<profile|Firefox::Marionette::Profile>,
+
 =head2 css
 
 accepts an L<element|Firefox::Marionette::Element> as the first parameter and a scalar CSS property name as the second parameter.  It returns the value of the computed style for that property.
@@ -1443,6 +1454,10 @@ causes the browser to traverse one step forward in the joint history of the curr
 
 returns the current active L<frame|Firefox::Marionette::Element> if there is one in the current browsing context.  Otherwise, this method returns undef.
 
+=head2 switch_to_shadow_root
+
+accepts an L<elemnet|Firefox::Marionette::Element> as a parameter and switches to it's L<shadow root|https://www.w3.org/TR/shadow-dom/>
+
 =head2 switch_to_frame
 
 accepts a L<frame|Firefox::Marionette::Element> as a parameter and switches to it within the current window.
